<?php

namespace Drupal\cards\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\TypedData\MapDataDefinition;
use Drupal\entityreference_view_mode\Plugin\Field\FieldType\EntityReferenceViewMode;

/**
 * Plugin implementation of the 'field_content_view' field type.
 *
 * @FieldType(
 *   id = "card_children_view",
 *   label = @Translation("Cards Children"),
 *   module = "cards",
 *   description = @Translation("Field referencing a piece of content and an associated view mode."),
 *   default_widget = "card_children_view_widget",
 *   default_formatter = "card_children_view_formatter"
 * )
 */
class CardChildrenView extends FieldItemBase {

    use CardViewTrait;


    /**
     * Build a list of view modes.
     *
     * @return array
     *   Array of view modes.
     */
    public static function availableViewModes() {
        $view_mode_ids = \Drupal::entityQuery('entity_view_mode')
            ->condition('targetEntityType', 'node')
            ->accessCheck(TRUE)
            ->execute();

        $view_mode_classes = \Drupal::service('entity_type.manager')->getStorage('entity_view_mode')->load($view_mode_ids);
        $view_modes = [];
        foreach ($view_mode_classes as $id => $class) {
            $view_modes[str_replace( 'node.', '', $id)] = $class->label();
        }
        return $view_modes;
    }

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings() {
    return [
      'widths' => [],
      'colors' => [],
        'view_modes' => [],
      'heights' => [],
      'icons' => [],
      'classes' => [],
    ];
  }

    public function fieldSettingsForm(array $form, FormStateInterface $form_state) {

        $element = parent::fieldSettingsForm($form, $form_state);

        $view_modes = $this->availableViewModes();
        $element['view_modes'] = [
            '#type' => 'checkboxes',
            '#title' => t('Enabled View Modes'),
            '#default_value' => $this->getSetting('view_modes'),
            '#description' => t('Select enabled view modes'),
            '#options' => $view_modes,
        ];

        $element += $this->addCardFieldSettingsForm($form, $form_state);

        return $element;
    }


    public static function schema(FieldStorageDefinitionInterface $field_definition) {

        $schema = CardChildrenView::addCardschema($field_definition);

        $schema['columns']['view_mode'] = [
            'type' => 'text',
            'size' => 'tiny',
            'not null' => FALSE,
        ];

        return $schema;

    }

    public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {



        $properties = CardChildrenView::addCardpropertyDefinitions($field_definition);

        $properties['view_mode'] = DataDefinition::create('string')
            ->setLabel(t('View Mode Value'));

        return $properties;

    }


}
